/*
io_x9_card.c

Hardware routines for interfacing with SD cards on a Ninjapass X9

Copyright (C) 2007 by CJ Bell
siegebell at gmail dot com


This software is provided 'as-is', without any express or implied warranty.  In
no event will the author be held liable for any damages arising from the use of
this software.  Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

 1. The origin of this software must not be misrepresented; you must not claim
    that you wrote the original software. If you use this software in a
    product, an acknowledgment in the product documentation would be
    appreciated but is not required.

 2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

 3. This notice may not be removed or altered from any source distribution.

*/


#include <nds.h>
//#include <stdio.h>
#include "io_x9_card.h"


//#define DO_DEBUG(statements) do { statements; } while(0)               
#define DO_DEBUG(statements)


// Shared data just to save space
static uint32 x9Buffer[0x200];

/* Writes data to the X9 until "not ready"
   In practice, will transfer 512 bytes
*/
void X9CardPolledWrite(uint32 flags, const uint32* buffer, const uint8* command)
{
    cardWriteCommand(command);
    CARD_CR2 = flags;

    do
    {
        if(CARD_CR2 & CARD_DATA_READY)
            CARD_DATA_RD = *buffer++;
    } while(CARD_CR2 & CARD_BUSY);
}

void X9CardIO(uint8 cmd, uint8 arg1, uint32 arg2, uint32 flags, uint32* buffer, uint32 words)
{
    uint8 command[8];
    command[7] = cmd;
    command[6] = (arg2>>24)&0xFF;
    command[5] = (arg2>>16)&0xFF;
    command[4] = (arg2>>8)&0xFF;
    command[3] = (arg2)&0xFF;
    command[2] = 0;
    command[1] = 0;
    command[0] = arg1;
    cardPolledTransfer(flags, buffer, words, command);
}

void X9CardWriteData(uint8 arg1, uint32 arg2, const void* buffer)
{
    uint8 command[8];
    command[7] = 0x64;
    command[6] = (arg2>>24)&0xFF;
    command[5] = (arg2>>16)&0xFF;
    command[4] = (arg2>>8)&0xFF;
    command[3] = (arg2)&0xFF;
    command[2] = 0;
    command[1] = 0;
    command[0] = arg1;
    X9CardPolledWrite(0xC1586000, buffer, command);
}

void cardIO_1()
{
    uint8 command[8] = {0x58, 0, 0, 0, 0, 0, 2, 0};
    cardPolledTransfer(0xA0586000, x9Buffer, 0, command);
}

void cardIO_2(uint32 arg)
{
    X9CardIO(0x5D, 0, arg, 0xA0586000, x9Buffer, 0);
}

void cardIO_3(uint8 arg0)
{
    uint8 command[8] = {0x58,0,0,0,0,0,arg0 | 2,0};

    uint32 var[0x80];
    cardPolledTransfer(0xA0586000, var, 0, command);
}

void cardIO_4(uint32 address)
{
    X9CardIO(0x5C, 0, address, 0xA0586000, x9Buffer, 0);
}

